/*********************************************************************************
**
** Description         : DS18x20 program with LCD Output
**                       show the temperatur at full celsius and
**                       the fraction in millicelsius*(10^-1)/625 (the 4 LS-Bits)
**
** Compiler            : AVR-GCC (WinAVR 20060421)
** Hardware            : AVR-Ctrl, DS18S20
** Chip type           : AT90S8535
** Clock frequency     : 8,000000 MHz
** Internal SRAM size  : 512
** External SRAM size  : 0
** Data Stack size     : 128
** Autor               : Michael Odenwald (odiwan@gmx.de)
** Date                : 20.06.2006
**
**********************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <string.h>

#include "lcd.h"
#include "onewire.h"
#include "crc8.h"
#include "delay.h"
#include "ds18x20.h"

void lcd_put_temp(const uint8_t subzero, uint8_t cel, uint8_t cel_frac_bits)
{
	uint8_t  outbuf[17];
	uint8_t  i;
	uint16_t decicelsius;

	lcd_gotoxy(0,1);

	// show "maximal" DS18S20 resulution
	sprintf(outbuf,"%c %i.%04i %cC",(subzero)?'-':'+', cel, cel_frac_bits*DS18X20_FRACCONV, 0xdf);

#if 0
	// show "rounding" DS18S20 resulution
	decicelsius = DS18X20_temp_to_decicel(subzero, cel, cel_frac_bits);
	sprintf(outbuf,"%c %i.%i %cC",(subzero)?'-':'+', decicelsius/10, decicelsius%10, 0xdf);
#endif

	for (i=strlen(outbuf); i<=15; i++)
	  outbuf[i] = ' ';
	outbuf[16] = '\0';

	lcd_puts(outbuf);
}

int main( void )
{
	uint8_t familycode;
	uint8_t subzero, cel, cel_frac_bits;

	lcd_init(LCD_DISP_ON);						// initialize display, cursor off
	lcd_puts_P("+++ AVR Ctrl +++");

	sei();

	while(1)	// main loop
	{

		if ( DS18X20_start_meas( DS18X20_POWER_PARASITE, NULL )
			== DS18X20_OK) {
			delay_ms(DS18B20_TCONV_12BIT);

			// read and show temparture
			familycode = DS18S20_ID; // for conversion routine
			if ( DS18X20_read_meas_single( familycode, &subzero,
					&cel, &cel_frac_bits) == DS18X20_OK ) {
				lcd_put_temp(subzero, cel, cel_frac_bits);
			}
			else
			{
				lcd_gotoxy(0,1);
				lcd_puts("CRC Error");
			}
		}
		else
		{
			lcd_gotoxy(0,1);
			lcd_puts("Start meas. fail");
		}

		delay_ms(1000);
	}
	return 0; /* never reached */
}
